"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.diff = diff;
exports.language = void 0;
exports.parse = parse;
exports.ready = ready;
exports.toString = toString;

var _style = require("./style");

var _wcwidth = _interopRequireDefault(require("wcwidth"));

var _runmode = _interopRequireDefault(require("codemirror/addon/runmode/runmode.node"));

require("codemirror/mode/meta");

var _css = require("css");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const modeMap = new Map();
const collisions = new Set();

for (const mode of _runmode.default.modeInfo) {
  const keys = new Set();

  for (const key of mode.alias ?? []) {
    keys.add(key);
  }

  if (keys.size === 0) {
    for (const key of mode.ext ?? []) {
      keys.add(key);
    }
  }

  for (const key of keys) {
    if (modeMap.has(key)) {
      collisions.add(key);
      modeMap.delete(key);
    } else if (!collisions.has(key)) {
      modeMap.set(key, mode);
    }
  }
}

const themes = new Map();

async function ready(options) {
  for (const lang of (options === null || options === void 0 ? void 0 : options.langs) ?? []) {
    const mode = modeMap.get(lang);

    if (mode == null) {
      throw new Error(`language ${mode} not found`);
    }

    await Promise.resolve(`codemirror/mode/${mode.mode}/${mode.mode}`).then(s => _interopRequireWildcard(require(s)));
  }

  themes.set('default', await getColorMap());

  for (const theme of (options === null || options === void 0 ? void 0 : options.themes) ?? []) {
    themes.set(theme, await getColorMap(theme));
  }
}

const handler = {
  get(_, language) {
    return (code, ...rest) => {
      return {
        language,
        spans: Array.from(code),
        nodes: rest
      };
    };
  }

};
const language = new Proxy({}, handler);
exports.language = language;

function colorOfRule(rule) {
  for (const item2 of rule.declarations ?? []) {
    if (item2.type === 'declaration') {
      const declaration = item2;

      if (declaration.property === 'color') {
        return declaration.value;
      }
    }
  }

  return undefined;
}

function* getTargetsOfRule(rule, theme = 'default') {
  for (const selector of rule.selectors ?? []) {
    const parts = selector.split(' ');

    if (parts.length === 1) {
      const [part] = parts;
      const classes = [];

      for (const match of part.matchAll(/\.([\w-]+)/g)) {
        const [, target] = match;
        classes.push(target);
      }

      if (classes.includes('CodeMirror')) {
        const classTheme = classes.find(c => /cm-s-[\w-]+/.test(c)) ?? 'default';

        if (classTheme === 'default' || classTheme === `cm-s-${theme}`) {
          yield 'fallback';
        }
      }
    }

    if (parts.length > 0 && parts[0] === `.cm-s-${theme}`) {
      if (parts.length !== 2) {
        throw new Error(`found css rule with more than two parts`);
      }

      const matched = parts[1].match(/\.cm-([\w-]+)\b/);

      if (matched == null) {
        continue;
      }

      const [, target] = matched;
      yield target;
    }
  }
}

async function getTheme(theme) {
  if (theme == null) {
    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
    // @ts-ignore
    return (await Promise.resolve().then(() => _interopRequireWildcard(require(`codemirror/lib/codemirror.css?raw`)))).default;
  } else {
    return (await Promise.resolve(`codemirror/theme/${theme}.css?raw`).then(s => _interopRequireWildcard(require(s)))).default;
  }
}

async function getColorMap(theme) {
  const css = (0, _css.parse)(await getTheme(theme));
  const map = new Map();

  for (const item of (css === null || css === void 0 ? void 0 : (_css$stylesheet = css.stylesheet) === null || _css$stylesheet === void 0 ? void 0 : _css$stylesheet.rules) ?? []) {
    var _css$stylesheet;

    if (item.type === 'rule') {
      const rule = item;
      const color = colorOfRule(rule);

      if (color != null) {
        for (const target of getTargetsOfRule(rule, theme)) {
          map.set(target, color);
        }
      }
    }
  }

  return map;
}

function getColor(style, options) {
  var _options$codeStyle, _options$codeStyle$fa;

  if (style == null) {
    style = 'fallback';
  }

  const override = (0, _style.getFromCodeStyle)(options === null || options === void 0 ? void 0 : options.codeStyle, style);

  if (override != null) {
    return override.text;
  }

  const theme = (options === null || options === void 0 ? void 0 : options.theme) ?? 'default';
  const themeRules = themes.get(theme);

  if (themeRules == null) {
    throw new Error(`you must first call \`await ready({ themes: [${theme}] })\``);
  }

  const fallbackOverride = options === null || options === void 0 ? void 0 : (_options$codeStyle = options.codeStyle) === null || _options$codeStyle === void 0 ? void 0 : (_options$codeStyle$fa = _options$codeStyle['fallback']) === null || _options$codeStyle$fa === void 0 ? void 0 : _options$codeStyle$fa.text;
  return themeRules.get(style) ?? fallbackOverride ?? themeRules.get('fallback') ?? '';
}

function parse(code, options) {
  const raw = integrate(reindent(code));
  const parsed = [];
  const mode = modeMap.get(code.language);

  if (mode == null) {
    throw new Error(`you must call \`await ready({ langs: ['${code.language}'] })\``);
  }

  _runmode.default.runMode(raw, mode.mime, (text, style) => {
    parsed.push({
      code: text,
      color: getColor(style, options)
    });
  });

  return parsed;
}

function toString(tokens) {
  return tokens.map(token => token.code).join('');
}

function reindent(code, indent = 0) {
  var _code$spans$at;

  if (typeof code === 'string') {
    if (code.at(0) !== '\n') {
      return ' '.repeat(indent) + code;
    }

    const regex = /^ +/gm;
    const matches = code.matchAll(regex);
    let min = Infinity;

    for (const match of matches) min = Math.min(min, match[0].length);

    const undent = new RegExp(`^ {${min}}`, 'gm');
    const result = code.substring(1).replace(undent, ' '.repeat(indent));
    return result;
  }

  if (((_code$spans$at = code.spans.at(0)) === null || _code$spans$at === void 0 ? void 0 : _code$spans$at.at(0)) !== '\n') return code;
  const regex = /\n */g;
  const matches = code.spans.flatMap(span => [...span.matchAll(regex)]);
  let min = Infinity;

  for (const [match] of matches) min = Math.min(min, match.length - 1);

  const undentRegex = new RegExp(`\n {${min}}`, 'g');
  const spans = code.spans.map(span => span.replace(undentRegex, '\n' + ' '.repeat(indent)));
  spans[0] = spans[0].substring(1);
  let index = 0;
  const nodes = [];

  for (const node of code.nodes) {
    const before = spans[index];
    const preindentRegex = /\n *$/;
    const indentation = before.match(preindentRegex);

    if (indentation != null) {
      spans[index] = spans[index].replace(preindentRegex, '\n');
      nodes.push(reindent(node, indentation[0].length - 1));
    } else {
      nodes.push(node);
    }

    index++;
  }

  return {
    language: code.language,
    spans,
    nodes
  };
}

function integrate(code) {
  if (typeof code === 'string') return code;
  return String.raw({
    raw: code.spans
  }, ...code.nodes.map(n => integrate(n)));
}

function isLevelEquivilant(one, two) {
  if (one === null || two === null) return false;

  if (typeof one === 'string' && typeof two !== 'string' || typeof one !== 'string' && typeof two === 'string') {
    return false;
  }

  if (typeof one === 'string' && typeof two === 'string') {
    return one === two;
  }

  if (typeof one !== 'string' && typeof two !== 'string') {
    if (one.spans.length !== two.spans.length) return false;
    return one.spans.every((span, i) => span === two.spans[i]);
  }

  throw new Error('Could not determine equivilance of nodes');
}

function chars(tokens) {
  return tokens.flatMap(({
    code,
    color
  }) => {
    return code.split('').map(c => ({
      code: c,
      color
    }));
  });
}

function tokens(chars) {
  const result = [];
  let token = null;

  for (const char of chars) {
    if (token == null) {
      token = char;
    } else {
      if (char.code === '\n') {
        result.push(token);
        token = null;
      } else if (token.color === char.color && token.morph === char.morph) {
        token.code += char.code;
      } else {
        result.push(token);
        token = char;
      }
    }
  }

  if (token != null) result.push(token);
  return result;
}

function diff(start, end, options) {
  start = reindent(start);
  end = reindent(end);
  const startParsed = chars(parse(start, options));
  const endParsed = chars(parse(end, options));
  let index = 0;
  let endex = 0;
  const result = [];

  function recurse(one, two) {
    const progress = (l, r) => {
      if (r == null) r = l;
      const startIndex = index;

      while (index < startIndex + l.length && index < startParsed.length) {
        result.push({
          code: startParsed[index].code,
          color: startParsed[index].color,
          morph: l === r ? 'retain' : 'delete'
        });
        index++;
      }

      const endIndex = endex;

      while (endex < endIndex + r.length && two !== '' && endex < endParsed.length) {
        if (r !== l) {
          result.push({
            code: endParsed[endex].code,
            color: endParsed[endex].color,
            morph: 'create'
          });
        }

        endex++;
      }
    };

    if (isLevelEquivilant(one, two)) {
      if (one == null || two == null) {
        throw new Error('equivilant nodes should not be null');
      }

      if (typeof one === 'string') {
        progress(one);
      } else {
        if (typeof two === 'string') throw new Error();

        for (let n = 0; n < one.nodes.length; n++) {
          progress(one.spans[n]);
          recurse(one.nodes[n], two.nodes[n]);
        }

        progress(one.spans.at(-1) ?? '');
      }
    } else {
      if (typeof one === 'string' && typeof two === 'string') {
        progress(one, two);
      } else if (typeof one === 'string') {
        progress(one, '');
      } else if (typeof two === 'string') {
        progress('', two);
      }

      if (one != null && typeof one !== 'string') {
        for (let n = 0; n < one.nodes.length; n++) {
          progress(one.spans[n], '');
          recurse(one.nodes[n], null);
        }

        progress(one.spans.at(-1) ?? '', '');
      }

      if (two != null && typeof two !== 'string') {
        for (let n = 0; n < two.nodes.length; n++) {
          progress('', two.spans[n]);
          recurse(null, two.nodes[n]);
        }

        progress('', two.spans.at(-1));
      }
    }
  }

  recurse(start, end);
  let [sat, sln, eat, eln] = [0, 0, 0, 0];
  const positioned = result.map(({
    code,
    color,
    morph
  }) => {
    const value = {
      code,
      color,
      morph,
      from: morph === 'create' ? null : [sat, sln],
      to: morph === 'delete' ? null : [eat, eln]
    };

    if (code === '\n') {
      if (morph !== 'create') {
        sat = 0;
        sln++;
      }

      if (morph !== 'delete') {
        eat = 0;
        eln++;
      }
    } else {
      if (morph !== 'create') {
        sat += (0, _wcwidth.default)(code);
      }

      if (morph !== 'delete') {
        eat += (0, _wcwidth.default)(code);
      }
    }

    return value;
  });
  const tokenized = tokens(positioned);
  return tokenized.filter(({
    code
  }) => {
    return code.length !== 0 && !/^ +$/.test(code);
  });
}