export declare type Colors = {
    text: string;
};
export declare type StyleOption = 'fallback' | 'keyword' | 'atom' | 'number' | 'def' | 'variable' | 'punctuation' | 'property' | 'operator' | 'variable-2' | 'variable-3' | 'type' | 'comment' | 'string' | 'string-2' | 'meta' | 'qualifier' | 'builtin' | 'bracket' | 'tag' | 'attribute' | 'hr' | 'link';
export declare function getFromCodeStyle(style: CodeStyle | null | undefined, option: StyleOption): Colors | null;
export interface CodeStyle {
    /**
     * The colors for any characters not covered by any other option.
     * ```ts
     * let a = 5; // both "=" and "5" will receive the `base` color
     * ```
     */
    fallback?: Colors;
    /**
     * The colors for a language keyword, like `function`.
     */
    keyword?: Colors;
    /**
     * The colors for a literal value, like "true" or "null".
     * ```ts
     * true
     * ```
     */
    atom?: Colors;
    /**
     * The colors for a number literal: "5".
     */
    number?: Colors;
    /**
     * The colors for an entity name.
     * ```ts
     * let foo = 5; // "foo" is the entity's name
     * ```
     */
    entityName?: Colors;
    /**
     * The colors for a string.
     * ```ts
     * 'some string'
     * ```
     */
    string?: Colors;
    /**
     * The color for a variable
     * ```ts
     * const what; // "what" is a variable
     * ```
     */
    variable?: Colors;
    /**
     * The colors for a code comment.
     * ```ts
     * // a comment
     * ```
     */
    comment?: Colors;
    /**
     * The colors for a regular expressions.
     */
    regexp?: Colors;
    /**
     * The colors for an operator.
     * ```ts
     * () => null // "=>" is an operator
     * ```
     */
    operator?: Colors;
}
//# sourceMappingURL=style.d.ts.map